
#ifndef CCAMERA_H
#define CCAMERA_H

#include "CVideo.h"
#include "CVector.h"
#include "fixed.h"
#include "CPolyObj.h"

#define TOP_Y     20
#define BOTTOM_Y  170
#define LEFT_X    20
#define RIGHT_X   300

enum eClipType { CT_LEFT,
		 CT_RIGHT,
		 CT_TOP,
		 CT_BOTTOM };

class CCamera {

public:
  CCamera( );
  CCamera( CVector vCenter, fixed p, fixed theta, fixed phi, fixed gamma, long eyedist );
  ~CCamera( );

  void changeViewingAngle( fixed theta, fixed phi = 0, fixed gamma = 0 );
  void changeViewingDistance( fixed p );
  void changeFocus( const CVector &vNewCenter );
  void changeEyeDistance( fixed newEyeDist );

  void shiftViewingAngle(fixed theta, fixed phi = 0, fixed gamma = 0);
  void shiftViewingDistance(fixed dp);

  // show contents of Camera view on buffer
  void view( CVideo & rVideo, const CPolyObj & PolyObj );

private:
  CVector m_vCenter;   // where the camera is looking
  fixed m_theta, m_phi, m_gamma;
  long m_p, m_eyedist;      // distance screen plane is from eye

  void reCalc(void);
  void convert3dPolyToScreenPoly( const CPoly & OldPoly, CPoly & NewPoly ) const;  
  void clipPolyScreen( const CPoly & OldPoly, CPoly & NewPoly ) const;
  bool convertWorldToScreen ( CVector vPt, long & ScreenX, long & ScreenY ) const;
  CVector convertWorldToCamWorld ( CVector vWorldPoint ) const;
  void clipPoly( const CPoly & OldPoly, CPoly & NewPoly, eClipType ClipType ) const;
};

#endif CCAMERA_H
