#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <process.h>
#include "CDb.h"
#include "debugmsg.h"
#include "extern.h"

CDb::CDb()
{
  DEBUGMSG("db: opening\n");

  if (entryExists(INDEX_ID) == false)
  {
    DEBUGMSG("db: creating new index entry\n");
    createIndex();
    createMain();
  }
}

CDb::~CDb()
{
  DEBUGMSG("db: closing\n");
}

void CDb::createIndex( void )
{
  CEntry *pIndexEntry;

  pIndexEntry = new CEntry(INDEX_SUBJECT_STR);
  pIndexEntry->m_Id = INDEX_ID;

  // create index text file
  FILE *fp = fopen(TEMP_FILE_NAME, "w");
  if (fp)
  {
    fputs(INDEX_TEXT_STR, fp);
    fclose(fp);
  }
  else
  {
    cerr << "Couldn't open " << TEMP_FILE_NAME << endl;
  }

  // write data to file now
  writeEntry(*pIndexEntry, TEMP_FILE_NAME);

  char szString[256];
  sprintf(szString, "del %s", TEMP_FILE_NAME);
  system(szString);
}

void CDb::createMain( void )
{
  CEntry *pMainEntry;

  pMainEntry = new CEntry(MAIN_SUBJECT_STR);
  pMainEntry->m_Id = MAIN_ID;

  // create index text file
  FILE *fp = fopen(TEMP_FILE_NAME, "w");
  if (fp)
  {
    fputs(MAIN_TEXT_STR, fp);
    fclose(fp);
  }
  else
  {
    cerr << "Couldn't open " << TEMP_FILE_NAME << endl;
  }

  // write data to file now
  writeEntry(*pMainEntry, TEMP_FILE_NAME);

  char szString[256];
  sprintf(szString, "del %s", TEMP_FILE_NAME);
  system(szString);

  // add main to index
  CEntry *pIndexEntry;
  pIndexEntry = getEntry(INDEX_ID);
  pIndexEntry->m_Header.m_Links.addLink(MAIN_ID);
  writeEntry(*pIndexEntry);
  delete pIndexEntry;
}

bool CDb::entryExists( long Id )
{
  char szFileName[256];
  getFileName(FT_HEADER, Id, szFileName);

  DEBUGMSG("db: entryExists: attempting to open file " << szFileName << endl);

  FILE *fp = fopen(szFileName, "rb");
  if (fp)
  {
    fclose(fp);
    return true;
  }
  else
    return false;
}

long CDb::convertToId( date date )
{
  long NewId;

  // Id = xxyyzzaa where xx = last 2 digits of year,
  //                     yy = month, zz = day, aa = entry number

  NewId = (date.da_year % 100) *    1000000L +   // xxxx
           date.da_mon         *      10000L +   // yy
           date.da_day         *        100L +   // zz
                     0L        *          1L;    // aa, refers to entry number

  DEBUGMSG("db: convertToId generated Id of " << NewId << endl);

  return NewId;
}

void CDb::getFileName( eFileType FileType, long Id, char *pszFileName )
{
  if (FileType == FT_HEADER)
  {
    sprintf(pszFileName, "headers\\%08ld.hdr", Id);
  }
  else if (FileType == FT_TEXT)
  {
    sprintf(pszFileName, "text\\%08ld.txt", Id);
  }
}

long CDb::getNewId( date date )
{
  long NewId;
  NewId = convertToId(date);

  for (int i = 0; i < MAX_ENTRIES_PER_DAY; i++, NewId++)
  {
    if (entryExists(NewId) == false)
      break;
  }

  DEBUGMSG("db: getnewid: newid is " << NewId << endl);

  return NewId;
}

long CDb::addEntry( date date, time time, char *szSubject, char *szFileName )
{
  CEntry *pNewEntry;

  pNewEntry = new CEntry(date, time, szSubject);
  long NewId = pNewEntry->m_Id = getNewId(date);

  writeEntry(*pNewEntry, szFileName);

  // update the index so it has the new entry linked onto it
  CEntry *pIndexEntry;
  pIndexEntry = getEntry(INDEX_ID);
  pIndexEntry->m_Header.m_Links.addLink(pNewEntry->m_Id);
  writeEntry(*pIndexEntry);

  delete pIndexEntry;
  delete pNewEntry;

  return NewId;
}

void CDb::writeHeader( long Id, CHeader & Header )
{
  char szFileName[256];
  getFileName(FT_HEADER, Id, szFileName);

  DEBUGMSG("db: writing header " << szFileName << endl);

  FILE *fp = fopen(szFileName, "wb");
  if (!fp)
  {
    cerr << "db: couldn't open file " << szFileName << endl;
    return;
  }

  fwrite(&Header.m_Date, sizeof(date), 1, fp);
  fwrite(&Header.m_Time, sizeof(time), 1, fp);
  fwrite(&Header.m_szSubject, sizeof(char), 256, fp);

  fwrite(&Header.m_Links.m_NumLinks, sizeof(int), 1, fp);
  fwrite(Header.m_Links.m_pIds, sizeof(long), Header.m_Links.m_NumLinks, fp);

  fclose(fp);
}

void CDb::writeEntry( CEntry & Entry, char *pszFileName )
{
  DEBUGMSG("db: writing entry " << Entry.m_Id << endl);

  writeHeader(Entry.m_Id, Entry.m_Header);

  if (pszFileName)  // copy file over too
  {
    char szNewFileName[256];
    sprintf(szNewFileName, "text\\%08ld.txt", Entry.m_Id);

    DEBUGMSG("copy file " << pszFileName << " to " << szNewFileName << endl);

    file_copy(pszFileName, szNewFileName);
  }
}

void CDb::readHeader( long Id, CHeader & Header )
{
  char szFileName[256];
  getFileName(FT_HEADER, Id, szFileName);

  DEBUGMSG("db: reading header " << szFileName << endl);

  FILE *fp = fopen(szFileName, "rb");
  if (!fp)
  {
    cerr << "db: couldn't open file " << szFileName << endl;
    return;
  }

  fread(&Header.m_Date, sizeof(date), 1, fp);
  fread(&Header.m_Time, sizeof(time), 1, fp);
  fread(&Header.m_szSubject, sizeof(char), 256, fp);

  fread(&Header.m_Links.m_NumLinks, sizeof(int), 1, fp);

  // read in links
  Header.m_Links.m_pIds = new long[Header.m_Links.m_NumLinks];
  fread(Header.m_Links.m_pIds, sizeof(long), Header.m_Links.m_NumLinks, fp);

  fclose(fp);
}

CEntry *CDb::getEntry( long Id )
{
  DEBUGMSG("db: getting entry " << Id << endl);

  CEntry *pNewEntry;
  pNewEntry = new CEntry(Id);
  readHeader(Id, pNewEntry->m_Header);

  return pNewEntry;
}

void CDb::getSubject( long Id, char *szSubject )
{
  CEntry *pTemp;
  pTemp = getEntry(Id);
  strcpy(szSubject, pTemp->m_Header.m_szSubject);
  delete pTemp;
}

void CDb::getSubject( long Id, char *szSubject, bool & HasLinks )
{
  CEntry *pTemp;
  pTemp = getEntry(Id);
  strcpy(szSubject, pTemp->m_Header.m_szSubject);
  if (pTemp->m_Header.m_Links.m_NumLinks != 0)
    HasLinks = true;
  else
    HasLinks = false;
  delete pTemp;
}
