#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <process.h>
#include <iostream.h>
#include "bool.h"
#include "debugmsg.h"
#include "CUi.h"

void CUi::CUi( CDb *pDb, CVideo *pVideo, long FirstId )
: m_pDb(pDb),
  m_WindowFocus(pVideo),
  m_History(FirstId),
	m_pCurEntry(0),
  m_pLinksUi(0),
  m_pVideo(pVideo)
{
  m_pVideo->CursOff();
  m_pVidBuffer = new CVidBuffer(m_pVideo, 0,0, 79,24);

  m_pVideo->clearArea(0,0, 79,24);

  m_pCurEntry = m_pDb->getEntry( FirstId );
  DEBUGMSG("cui: Entry Id is " << m_pCurEntry->m_Id << endl);
  m_pLinksUi = new CLinksUi(m_pCurEntry, pDb, pVideo);

  m_WindowFocus.showFocus();
}

void CUi::~CUi()
{
	delete m_pCurEntry;
	delete m_pLinksUi;

  delete m_pVidBuffer;
  m_pVideo->CursOn();
}

eInput CUi::getInput( bool allowModify )
{
	eInput returnValue = I_INVALID;
	char cInput;

	do
	{
		cInput = getch();

		switch (cInput)
		{
			case 0 :	// arrow key
				switch(getch())
				{
					case K_DOWN :
						if (m_WindowFocus.m_Focus == F_LINKS)
							returnValue = I_LINKS_DOWN;
						else if (m_WindowFocus.m_Focus == F_TEXT)
							returnValue = I_TEXT_DOWN;
						break;
					case K_UP :
						if (m_WindowFocus.m_Focus == F_LINKS)
							returnValue = I_LINKS_UP;
						else if (m_WindowFocus.m_Focus == F_TEXT)
							returnValue = I_TEXT_UP;
						break;
          case K_LEFT :
						returnValue = I_HISTORY_BACK;
						break;
          case K_RIGHT :
            if (m_WindowFocus.m_Focus == F_LINKS)
              returnValue = I_LINKS_FOLLOW;
						break;
          case K_ALTRIGHT :
            returnValue = I_HISTORY_FORWARD;
            break;
          case K_SHIFTTAB :
            returnValue = I_FOCUS_PREVIOUS;
            break;
					case K_DEL :
            if (allowModify)
              if (m_WindowFocus.m_Focus == F_LINKS)
                returnValue = I_LINKS_DELETE;
						break;
          case K_CTRL_INS :
            if (allowModify)
              returnValue = I_LINKS_INSERT;
            break;
          case K_INS :
            if (allowModify && m_WindowFocus.m_Focus == F_LINKS)
              returnValue = I_LINKS_CREATE;
            else if (allowModify && m_WindowFocus.m_Focus == F_TEXT)
              returnValue = I_CREATE_ENTRY;
						break;
          case K_F1 :
            returnValue = I_GOTO_INDEX;
            break;
          case K_F2 :
            returnValue = I_GOTO_MAIN;
            break;
          case K_ALT_X :
            returnValue = I_QUIT;
            break;
          case K_PGUP :
            if (m_WindowFocus.m_Focus == F_LINKS)
              returnValue = I_LINKS_PGUP;
            break;
          case K_PGDN :
            if (m_WindowFocus.m_Focus == F_LINKS)
              returnValue = I_LINKS_PGDN;
            break;
          case K_HOME :
            if (m_WindowFocus.m_Focus == F_LINKS)
              returnValue = I_LINKS_HOME;
            break;
          case K_END :
            if (m_WindowFocus.m_Focus == F_LINKS)
              returnValue = I_LINKS_END;
            break;
          case K_ALT_J :
            returnValue = I_JUMP_DOS;
            break;
				}
				break;
      case K_TAB :
        returnValue = I_FOCUS_NEXT;
        break;
      case K_SPACE :
        if (allowModify == false)
          returnValue = I_CHOOSE_ENTRY;
        break;
			case K_ENTER :
        if (allowModify)
          returnValue = I_TEXT_EDIT;
				break;
      case K_ESC :
        returnValue = I_QUIT;
				break;
		}
	} while (returnValue == I_INVALID);

	return returnValue;
}

long CUi::createNewEntry( void )
{
  DEBUGMSG("cui: createnewentry\n");

  date date;
  getdate(&date);
  time time;
  gettime(&time);

  char szSubject[256];
  strcpy(szSubject, "");
  m_pVideo->boxInputString(szSubject, 60, "Please enter a subject.", "Subject");

  if (szSubject[0] == 0)
    return -1;

  CVidBuffer *pVidBuffer;
  pVidBuffer = new CVidBuffer(m_pVideo, 0,0, 79,24);
  
  // edit tmp.fil via q.exe
  spawnl(P_WAIT, EDITOR_COMMAND, EDITOR_COMMAND, TEMP_FILE_NAME, NULL);
  
  delete pVidBuffer;
  
  long ReturnId = m_pDb->addEntry(date, time, szSubject, TEMP_FILE_NAME);
  
  char szString[256];
  sprintf(szString, "del %s", TEMP_FILE_NAME);
  system(szString);

  DEBUGMSG("cui: returnid is " << ReturnId << endl);

  return ReturnId;
}

long CUi::chooseEntry( bool allowModify )
{
  bool done = false, changeEntry;
  bool dirtyLinks = false, dirtyText = false;
  bool moveForward, changeLink;
  eInput Input;
  long EntryId;
  int LinkNum;
  CHistoryData HistoryData;
  CEntry *pCurEntry = 0;
  CTextUi *pTextUi = 0;
  CHeaderUi *pHeaderUi = 0;
  long CurId;

  m_pVideo->clearArea(0,79, 0, 24);
  
  if (allowModify == false)
    m_pVideo->displayPaddedString(" Please choose an entry to link.", 0, 24, 80, (7<<4));
  else
    m_pVideo->displayPaddedString(" Ins = Add Entry | C-Ins = Add Link | Del = Delete Link | Esc = Quit ", 0, 24, 80, (7<<4));

  // initialize display of current entry
  CurId = m_pCurEntry->m_Id;
  pCurEntry = m_pDb->getEntry( CurId );
  pTextUi = new CTextUi(pCurEntry, m_pDb, m_pVideo);
  pHeaderUi = new CHeaderUi(pCurEntry, m_pVideo);
  pTextUi->display();
  pHeaderUi->display();
  m_pLinksUi->display();

  while (!done)
    {
      m_WindowFocus.showFocus();
      
      Input = getInput(allowModify);
      
      changeEntry = false;
      changeLink = false;
      moveForward = false;
      LinkNum = 0;
      
      switch (Input)
	{
	case I_FOCUS_NEXT :
	  m_WindowFocus.next();
	  break;
	case I_FOCUS_PREVIOUS :
	  m_WindowFocus.previous();
	  break;
	case I_HISTORY_BACK :
	  changeEntry = true;
	  m_History.back( m_pLinksUi->m_CurLink );
	  HistoryData = m_History.getCurHistory();
	  EntryId = HistoryData.m_Id;
	  LinkNum = HistoryData.m_LinkNum;
	  changeLink = true;
	  break;
	case I_HISTORY_FORWARD :
	  changeEntry = true;
	  m_History.forward( m_pLinksUi->m_CurLink );
	  HistoryData = m_History.getCurHistory();
	  EntryId = HistoryData.m_Id;
	  LinkNum = HistoryData.m_LinkNum;
	  changeLink = true;
	  break;
	case I_LINKS_UP :
	  m_pLinksUi->up();
	  changeLink = true;
	  break;
	case I_LINKS_DOWN :
	  m_pLinksUi->down();
	  changeLink = true;
	  break;
	case I_LINKS_PGUP :
	  m_pLinksUi->pgup();
	  changeLink = true;
	  break;
	case I_LINKS_PGDN :
	  m_pLinksUi->pgdn();
	  changeLink = true;
	  break;
	case I_LINKS_HOME :
	  m_pLinksUi->home();
	  changeLink = true;
	  break;
	case I_LINKS_END :
	  m_pLinksUi->end();
	  changeLink = true;
	  break;
	case I_LINKS_DELETE :
	  m_pLinksUi->deleteLink();
	  changeLink = true;
	  dirtyLinks = true;
	  break;
	case I_LINKS_INSERT :
	  CUi *tmp_ui;
	  
	  tmp_ui = new CUi(m_pDb, m_pVideo, MAIN_ID);
	  
	  EntryId = tmp_ui->chooseEntry(false);
	  delete tmp_ui;
	  
	  if (EntryId > 0)
	    {
	      if (m_pCurEntry->m_Id != INDEX_ID)
		{
		  m_pLinksUi->insertLink(EntryId);
		  dirtyLinks = true;
		}
	      else  // no need to insert for index since it's automatic
		{
		  // instead, just reload it
		  delete m_pCurEntry;
		  delete m_pLinksUi;
		  
		  m_pCurEntry = m_pDb->getEntry( INDEX_ID );
		  m_pLinksUi = new CLinksUi(m_pCurEntry, m_pDb, m_pVideo, LinkNum);
		  m_pLinksUi->end();
		  m_pLinksUi->display();
		}
	      changeLink = true;
	    }
	  break;
	case I_LINKS_CREATE :
	  EntryId = createNewEntry();
	  
	  if (EntryId > 0)
	    {
	      if (m_pCurEntry->m_Id != INDEX_ID)
		{
		  m_pLinksUi->insertLink(EntryId);
		  dirtyLinks = true;
		}
	      else  // no need to insert for index since it's automatic
		{
		  // instead, just reload it
		  delete m_pCurEntry;
		  delete m_pLinksUi;
		  
		  m_pCurEntry = m_pDb->getEntry( INDEX_ID );
		  m_pLinksUi = new CLinksUi(m_pCurEntry, m_pDb, m_pVideo, LinkNum);
		  m_pLinksUi->end();
		  m_pLinksUi->display();
		}
	      changeLink = true;
	    }
	  break;
	case I_LINKS_FOLLOW :
	  EntryId = m_pLinksUi->follow();
	  if (EntryId > 0)
	    {
	      m_History.setCurLink( m_pLinksUi->m_CurLink );
	      changeEntry = true;
	      moveForward = true;
	      changeLink = true;
	    }
	  break;
	case I_TEXT_UP :
	  pTextUi->up();
	  break;
	case I_TEXT_DOWN :
	  pTextUi->down();
	  break;
	case I_TEXT_EDIT :
	  pHeaderUi->edit();
	  pTextUi->edit();
	  dirtyText = true;
	  break;
	case I_CREATE_ENTRY :
	  EntryId = createNewEntry();
	  if (EntryId > 0)
	    {
	      m_History.setCurLink( m_pLinksUi->m_CurLink );
	      changeEntry = true;
	      moveForward = true;
	      changeLink = true;
	    }
	  break;
	case I_QUIT :
	  if (allowModify)
	    {
	      if (m_pVideo->YesNo("Really Quit?"))
		done = true;
	    }
	  else
	    {
	      if (m_pVideo->YesNo("Cancel?"))
		done = true;
	    }
	  break;
	case I_CHOOSE_ENTRY :
	  delete pCurEntry;
	  delete pTextUi;
	  delete pHeaderUi;
	  
	  if (m_pLinksUi->m_CurLink > 0)
	    CurId = m_pLinksUi->m_pLinks->m_pIds[m_pLinksUi->m_CurLink-1];
	  else
	    CurId = m_pCurEntry->m_Id;
	  
	  return CurId;
	case I_GOTO_INDEX :
	  m_History.setCurLink( m_pLinksUi->m_CurLink );
	  EntryId = INDEX_ID;
	  changeEntry = true;
	  moveForward = true;
	  changeLink = true;
	  break;
	case I_GOTO_MAIN :
	  m_History.setCurLink( m_pLinksUi->m_CurLink );
	  EntryId = MAIN_ID;
	  changeEntry = true;
	  moveForward = true;
	  changeLink = true;
	  break;
	case I_JUMP_DOS :
	  CVidBuffer *pVidBuffer;
	  pVidBuffer = new CVidBuffer(m_pVideo, 0,0, 79,24);
	  
	  spawnl(P_WAIT, "command.com", "command.com", NULL);
	  
	  delete pVidBuffer;
	  break;
	default :
	  assert(0);
	  break;
	}
      
      if (changeLink)
	{
	  // redisplay current entry linked to
	  if (m_pLinksUi->m_CurLink > 0)
	    CurId = m_pLinksUi->m_pLinks->m_pIds[m_pLinksUi->m_CurLink-1];
	  else
	    CurId = m_pCurEntry->m_Id;
	  
	  delete pCurEntry;
	  delete pTextUi;
	  delete pHeaderUi;
	  
	  pCurEntry = m_pDb->getEntry( CurId );
	  pTextUi = new CTextUi(pCurEntry, m_pDb, m_pVideo);
	  pHeaderUi = new CHeaderUi(pCurEntry, m_pVideo);
	  
	  pTextUi->display();
	  pHeaderUi->display();
	}
      
      if (dirtyText)
	{
	  m_pDb->writeEntry(*pCurEntry);
	  dirtyText = false;
	  pTextUi->display();
	}
      if (dirtyLinks)
	{
	  m_pDb->writeEntry(*m_pCurEntry);
	  m_pLinksUi->display(true);
	  dirtyLinks = false;
	}
      
      if (changeEntry && EntryId != m_pCurEntry->m_Id)
	{
	  // delete old displayed stuff
	  delete pCurEntry;
	  delete pTextUi;
	  delete pHeaderUi;
	  
	  // delete old current entry
	  delete m_pCurEntry;
	  delete m_pLinksUi;
	  
	  m_pCurEntry = m_pDb->getEntry( EntryId );
	  m_pLinksUi = new CLinksUi(m_pCurEntry, m_pDb, m_pVideo, LinkNum);
	  m_pLinksUi->display();
	  
	  if (m_pLinksUi->m_CurLink > 0)
	    CurId = m_pLinksUi->m_pLinks->m_pIds[m_pLinksUi->m_CurLink-1];
	  else
	    CurId = m_pCurEntry->m_Id;
	  pCurEntry = m_pDb->getEntry( CurId );
	  pTextUi = new CTextUi(pCurEntry, m_pDb, m_pVideo);
	  pHeaderUi = new CHeaderUi(pCurEntry, m_pVideo);
	  
	  pTextUi->display();
	  pHeaderUi->display();
	  
	  if (moveForward)
	    m_History.addHistory(m_pCurEntry->m_Id, 0);
	}
    }
  
  delete pCurEntry;
  delete pTextUi;
  delete pHeaderUi;

  return -1;
}
